#!/bin/bash

# Copyright (c) 2022 Vossloh-Schwabe Deutschland GmbH
# Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the “Software”), to deal 
# in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of 
# the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:
# The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.
# THE SOFTWARE IS PROVIDED “AS IS”, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
# WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS
# OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR
# OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.


LANGW_WHL="install_files/vs_lan_gateway_server-1.15-py3-none-any.whl"

SCRIPT_PATH=$(dirname "$(realpath -s "$0")")
cd $SCRIPT_PATH
err () {
    echo "Error in $(caller)"
    exit 1
}
trap 'err' err
if [[ $EUID -ne 0 ]]; then
    echo "Has to be run as root"
    exit 1
fi

join_arr() {
    local IFS="$1"
    shift
    echo "$*"
}

stringContain() {
    [ -z "$1" ] || { [ -z "${2##*$1*}" ] && [ -n "$2" ];};
}

service_exists() {
    local n=$1
    if [[ $(systemctl list-units --all -t service --full --no-legend "$n.service" | sed 's/^\s*//g' | cut -f1 -d' ') == $n.service ]]; then
        return 0
    else
        return 1
    fi
}

json=""
dbms=none
config_file=/var/vs_lan_gateway_server/config.json
config_path_old=/root/.vs_lan_gateway_server
config_file_old=/root/.vs_lan_gateway_server/config.json
config_path_new=/var/vs_lan_gateway_server
config_file_new=/var/vs_lan_gateway_server/config.json

echo 'Check for running lan-gateway service'
if service_exists lan-gateway; then
	echo 'stopping the running lan-gateway service'
    /bin/systemctl stop lan-gateway
fi

if [ -e $config_file_old ] || [ -e $config_file_new ]; then
    echo 'The Configuration file is already existing. You can use "update.sh" to keep it'
    config_answer=o
    while [ $config_answer != "n" -a $config_answer != "y" ]; do
        read -p "Do you really want to create new configuration? ((y)es/(n)o): " config_answer
    done
    if [ $config_answer = "n" ]; then
        echo ' '
        echo ' '
        echo '###########################################################'
        echo '# Setup stopped.                                          #'
        echo '# Please use "update.sh" to update without config change. #'
        echo '###########################################################'
        exit 1
    fi
fi

mkdir -p $config_path_new

echo 'Database type selection'

while [ $dbms != "influxdb" -a $dbms != "mariadb" -a $dbms != "i" -a $dbms != "m" ]; do
    read -p "Enter the database system you want to use ((i)nfluxdb/(m)ariadb): " dbms
done

if [ $dbms = "i" ]; then
    dbms=influxdb
    packages=$dbms
    py_pack=$dbms
elif [ $dbms = "m" ]; then
    dbms=mariadb
    packages="mariadb-server libmariadb-dev"
    py_pack="mariadb==1.0.11"
fi

mkdir -p /etc/grafana/provisioning/datasources
mkdir -p /etc/grafana/provisioning/dashboards

if [ ! -e /etc/apt/sources.list.d/influxdb.list ] && [ $dbms = "influxdb" ]; then
    curl https://repos.influxdata.com/influxdb.key | gpg --dearmor | sudo tee /usr/share/keyrings/influxdb-archive-keyring.gpg >/dev/null
    echo "deb [signed-by=/usr/share/keyrings/influxdb-archive-keyring.gpg] https://repos.influxdata.com/debian $(lsb_release -cs) stable" | sudo tee /etc/apt/sources.list.d/influxdb.list
fi

if [ ! -e /etc/apt/sources.list.d/grafana.list ]; then
    curl https://packages.grafana.com/gpg.key | gpg --dearmor | sudo tee /usr/share/keyrings/grafana-archive-keyrings.gpg >/dev/null
    echo "deb [signed-by=/usr/share/keyrings/grafana-archive-keyrings.gpg] https://packages.grafana.com/oss/deb stable main" | sudo tee /etc/apt/sources.list.d/grafana.list
fi
echo 'Raspberry Pi update'
apt-get update
apt-get -y upgrade
echo 'Package install'
apt-get install -y python3-pip python3-werkzeug adduser libfontconfig1 libssl-dev $packages grafana
python3 -m pip install -I PyMySQL
python3 -m pip install -I sslpsk2
python3 -m pip install -I $LANGW_WHL
python3 -m pip install -U $py_pack


if [ $dbms = "mariadb" ]; then
    read -p "Enter the database name: " mariadb_dbname
    mariadb_username="root"
    while [ $mariadb_username = "root" ]; do
        read -p "Enter the database username (root cannot be used): " mariadb_username
    done
    read -sp "Enter the database password: " mariadb_password
    cat ./install_files/mariadb.yaml | sed -e "s/<database>/$mariadb_dbname/" -e "s/<username>/$mariadb_username/" -e "s/<password>/$mariadb_password/" > /etc/grafana/provisioning/datasources/mariadb.yaml
    echo
    json+="{\"mariadb\": {\"enabled\": true, \"username\": \"$mariadb_username\", \"dbname\": \"$mariadb_dbname\", \"password\": \"$mariadb_password\"}"
    echo 'Configuring MariaDB database system'
    MY_CNF_CREATED=0
    if [ $mariadb_dbname ]; then
        if [ -e ~/.my.cnf ]; then
            MY_CNF_CREATED=1
            mv ~/.my.cnf ~/.my.cnf.000001
        fi
        cat > ~/.my.cnf <<EOF
[mysql]
user=root
password=
EOF
        cat install_files/init.sql | sed -e "s/<database>/$mariadb_dbname/" -e "s/<username>/$mariadb_username/" -e "s/<password>/$mariadb_password/" | mysql
        rm ~/.my.cnf
        if [ $MY_CNF_CREATED -eq 1 ]; then
            mv ~/.my.cnf.000001 ~/.my.cnf
        fi
    fi  
else
    cp install_files/influxdb.yaml /etc/grafana/provisioning/datasources/
fi

echo 'Database Setup done'

echo 'Please enter credentials for the web configuration'
read -p 'Username: ' web_username
read -sp 'Password: ' web_password
web_password=`python3 -c "from werkzeug.security import generate_password_hash; print(generate_password_hash(\"$web_password\"))"`

if [ ! -z "$json" ]; then
    json+=", "
else
    json+="{"
fi
json+="\"web\": {\"username\": \"$web_username\", \"password\": \"$web_password\"}"

json+="}"
echo $json > $config_file_new

echo 'Setting up lan-gateway service configuration file'
cp install_files/lan-gateway.service /etc/systemd/system/lan-gateway.service
chmod 644 /etc/systemd/system/lan-gateway.service

grafana-cli plugins install ryantxu-ajax-panel

cp install_files/dashboard.yaml /etc/grafana/provisioning/dashboards/

mkdir -p $config_path_new/dashboards
chmod 755 $config_path_new
chmod 755 $config_path_new/dashboards

echo 'Starting Grafana'
/bin/systemctl daemon-reload
/bin/systemctl enable grafana-server
/bin/systemctl start grafana-server

echo ' '
echo ' '
echo '################################################################################################'
echo '# Setup successful.                                                                            #'
echo '# To enable starting the server at boot, run "sudo systemctl enable lan-gateway"               #'
echo '# To start the server, run "sudo systemctl start lan-gateway"                                  #'
echo '# The configuration website is reachable at following addresses:                               #'
for a in `hostname -I`; do
    if stringContain ':' $a; then
        echo "  http://[$a]:31460/"
    else
        echo "  http://$a:31460/"
    fi
done
echo '# Configuration website after starting: http://<ip of device>:31460/                           #'
echo '# Some additional configurations on the configuration web page are needed                      #'
echo '# before the server can work completely.                                                       #'
echo '################################################################################################'
